#!/bin/sh
########### original script makerpms.sh from samba distribution ###########
# Copyright (C) John H Terpstra 1998-2002
# Updated for RPM 3 by Jochen Wiedmann, joe@ispsoft.de
# Changed for a generic tar file rebuild by abartlet@pcug.org.au
# Changed by John H Terpstra to build on RH7.2 - should also work for earlier versions jht@samba.org
# Changed by Gratien D'haese to work with mkCDrec project

# The following allows environment variables to override the target directories
#   the alternative is to have a file in your home directory calles .rpmmacros
#   containing the following:
#   %_topdir  /home/mylogin/RPM
#   %__check_files         /usr/lib/rpm/check-files %{buildroot}
#   %_unpackaged_files_terminate_build      1
#   %_missing_doc_files_terminate_build     1
#
# Note: Under this directory rpm expects to find the directories:
#   ~/RPM/BUILD/  RPMS/  SOURCES/  SPECS/  SRPMS/
#   ~/RPM/RPMS/i386 ~/RPM/RPMS/i686

if [ -f ~/.rpmmacros ]; then
	TOPDIR=`awk '/topdir/ {print $2}' < ~/.rpmmacros`
	if [ "z$TOPDIR" <> "z" ]; then
		SPECDIR=${TOPDIR}/SPECS
		SRCDIR=${TOPDIR}/SOURCES
	fi
fi

SPECDIR=${SPECDIR:-/usr/src/RPM/SPECS}
SRCDIR=${SRCDIR:-/usr/src/RPM/SOURCES}

#########################
#	Subroutines	#
#########################

echo () {
    if [ "X$1" = "X-n" ]; then
        shift
        printf "%s" "$*"
    else
        printf "%s\n" "$*"
    fi
}

which_distro () {
    os="`uname -s`"
    arch="`uname -m`"

    wd_result () {
        echo "$1"
        exit
    }

    wd_check () {
        grep "$1" "$2" >/dev/null 2>&1
        if [ $? = 0 ]; then
            wd_result "$3"
        fi
    }

    case $os in
        SunOS)
            if [ "$arch" != "sun4u" ]; then
                wd_result unknown
            fi

            release="`uname -r`"

            case $release in
                5.7 | 5.7.*)
                    wd_result solaris-7-sun4
                    ;;

                5.8 | 5.8.*)
                    wd_result solaris-8-sun4
                    ;;

                *)
                    wd_result unknown
                    ;;
            esac
            ;;

        Linux)
            case $arch in
                i[34567]86)
                    # Linux Mandrake
                    if [ -f /etc/mandrake-release ]; then
                        wd_check "8.0" /etc/mandrake-release mandrake-80-i586
                        wd_check "8.1" /etc/mandrake-release mandrake-81-i586
                        wd_check "8.2" /etc/mandrake-release mandrake-82-i586
                        wd_check "9.0" /etc/mandrake-release mandrake-90-i586
                        wd_check "9.1" /etc/mandrake-release mandrake-91-i586
                        wd_check "10.0" /etc/mandrake-release mandrake-100-i586
                    fi

                    # SuSE Linux
                    if [ -f /etc/SuSE-release ]; then
                        wd_check "7.1" /etc/SuSE-release suse-71-i386
                        wd_check "7.2" /etc/SuSE-release suse-72-i386
                        wd_check "7.3" /etc/SuSE-release suse-73-i386
                        wd_check "8.0" /etc/SuSE-release suse-80-i386
                        wd_check "8.1" /etc/SuSE-release suse-81-i386
                        wd_check "9.1" /etc/SuSE-release suse-91-i386
                    fi

                    # Red Hat Linux
                    if [ -f /etc/redhat-release ]; then
                        wd_check "6.0" /etc/redhat-release redhat-60-i386
                        wd_check "6.1" /etc/redhat-release redhat-61-i386
                        wd_check "6.2" /etc/redhat-release redhat-62-i386
                        wd_check "7.0" /etc/redhat-release redhat-70-i386
                        wd_check "7.1" /etc/redhat-release redhat-71-i386
                        wd_check "7.2" /etc/redhat-release redhat-72-i386
                        wd_check "7.3" /etc/redhat-release redhat-73-i386
                        wd_check "8.0" /etc/redhat-release redhat-80-i386
                        wd_check "9" /etc/redhat-release redhat-90-i386
                    fi

                    # Debian GNU/Linux
                    if [ -f /etc/debian_version ]; then
                        wd_check "2.2" /etc/debian_version debian-potato-i386
                        wd_check "3.0" /etc/debian_version debian-woody-i386
                    fi

		    # Fedora Core
		    if [ -f /etc/fedora-release ]; then
			wd_check "1" /etc/fedora-release fedora-core-1-i386
			wd_check "2" /etc/fedora-release fedora-core-2-i386
			wd_check "3" /etc/fedora-release fedora-core-3-i386
			wd_check "4" /etc/fedora-release fedora-core-4-i386
		    fi

                    # Must not be recognized
                    wd_result unknown
                    ;;

                ppc)
                    # Yellow Dog Linux Champion Server
                    if [ -f /etc/yellowdog-release ]; then
                        wd_check "2.0" /etc/yellowdog-release yellowdog-20-ppc
                        wd_check "2.1" /etc/yellowdog-release yellowdog-21-ppc
                        wd_check "2.2" /etc/yellowdog-release yellowdog-22-ppc
                    fi

		    # Debian GNU/Linux
		    if [ -f /etc/debian_version ]; then
			wd_check "2.2" /etc/debian_version debian-potato-ppc
                        wd_check "3.0" /etc/debian_version debian-woody-ppc
                    fi

                    # Must not be recognized
                    wd_result unknown
                    ;;
		ia64)
		   #Red hat Linux
			if [ -f /etc/redhat-release ]; then
                        	wd_check "Enterprise" /etc/redhat-release redhat-Enterprise-ia64
                    	fi
		   # Must not be recognized
			wd_result unknown
		   ;;
		x86_64)
		   # Fedora Core
		    if [ -f /etc/fedora-release ]; then
			wd_check "3" /etc/fedora-release fedora-core-3-x86_64
			wd_check "4" /etc/fedora-release fedora-core-4-x86_64
		    fi
		    # Must not be recognized
                    wd_result unknown
		    ;;
		sparc)
		   # Sparc/Linux
		    if [ -f /etc/redhat-release ]; then
			wd_check "7" /etc/redhat-release redhat-7-sparc
		    fi
		    ;;
                *)
                    # Strange architecture
                    wd_result unknown
                    ;;
            esac
            ;;

        *)
            # Strange OS
            wd_result unknown
            ;;
    esac
}

cleanup () {
    echo
    printf "Cleaning up temporary files... "
    #rm -f $INSTALLER
    #rm -f $SNARF
    echo "done." 
    echo
}
# a few standard error conditions

# a generic error
#
# $1: the error message
bail_error () {
    echo Error!
    echo
    echo $1
    echo

    cleanup
    exit 1
}
# the os isn't supported
bail_os () {
    echo Error!
    echo
    echo Sorry, your operating system is not supported by mkCDrec.
    echo

    cleanup
    exit 1
}   
# the recognized distribution isn't supported
bail_unsupported () {
    echo Error!
    echo
    echo Sorry, the operating system $1 is not supported by mkCDrec.
    echo

    cleanup
    exit 1
}

# the architecture isn't supported
#
# $1: the architecture name (uname -m)
bail_cpu () {
    echo Error!
    echo
    echo Sorry, mkCDrec is not available for computers of type $1.
    echo
    
    cleanup
    exit 1
}   

#########################
#	MAIN CODE	#
#########################
# At this point the SPECDIR and SRCDIR vaiables must have a value!

USERID=`id -u`
GRPID=`id -g`
VERSION="`cat ../VERSION | cut -d_ -f 2`"

echo "==> Autodetecting system type..."

os=`uname -s`
if [ $os != Linux -a $os != SunOS ]
then
    bail_os $os
fi

TPROC=`uname -m`

case $TPROC in
    ppc)
        PROC=ppc
	BOOTARCH=powermac
        ;;
    i[34567]86)
        PROC=i386
	BOOTARCH=x86
        ;;
    sun4*)
        PROC=sun4
        ;;
    ia64*)
    	PROC=ia64
	BOOTARCH=ia64
	;;
    x86_64)
	PROC=x86_64
	BOOTARCH=x86_64
	;;
    sparc)
	PROC=sparc
	BOOTARCH=sparc
	;;
    *)
        bail_cpu $TPROC
        ;;
esac

DISTRO_STRING=`which_distro`

case $DISTRO_STRING in
    solaris-7-sun4 | solaris-8-sun4)
        DISTRO_EQUIV='solaris-7-sun4'
        ;;
    mandrake-80-i586)
        DISTRO_EQUIV='mandrake-80-i586'
        ;;
    mandrake-81-i586)
        DISTRO_EQUIV='mandrake-81-i586'
        ;;
    mandrake-82-i586)
        DISTRO_EQUIV='mandrake-82-i586'
        ;;
    mandrake-90-i586)
        DISTRO_EQUIV='mandrake-90-i586'
        ;;
    mandrake-91-i586)
        DISTRO_EQUIV='mandrake-91-i586'
        ;;
    mandrake-100-i586)
        DISTRO_EQUIV='mandrake-100-i586'
        ;;
    suse-71-i386)
        DISTRO_EQUIV='suse-71-i386'
        ;;
    suse-72-i386)
        DISTRO_EQUIV='suse-72-i386'
        ;;
    suse-73-i386)
        DISTRO_EQUIV='suse-73-i386'
        ;;
    suse-80-i386)
        DISTRO_EQUIV='suse-80-i386'
        ;;
    suse-91-i386)
        DISTRO_EQUIV='suse-91-i386'
        ;;
    redhat-62-i386)
        DISTRO_EQUIV='redhat-62-i386'
        ;;
    redhat-70-i386)
        DISTRO_EQUIV='redhat-70-i386'
        ;;
    redhat-71-i386)
        DISTRO_EQUIV='redhat-71-i386'
        ;;
    redhat-72-i386)
        DISTRO_EQUIV='redhat-72-i386'
        ;;
    redhat-73-i386)
        DISTRO_EQUIV='redhat-73-i386'
        ;;
    redhat-80-i386)
	DISTRO_EQUIV='redhat-80-i386'
	;;
    redhat-90-i386)
	DISTRO_EQUIV='redhat-90-i386'
	;;
    fedora-core-1-386)
	DISTRO_EQUIV='fedore-core-1-386'
	;;
    fedora-core-2-i386)
	DISTRO_EQUIV='fedore-core-2-i386'
	;;
    fedora-core-3-i386)
	DISTRO_EQUIV='fedore-core-3-i386'
	;;
    fedora-core-4-i386)
	DISTRO_EQUIV='fedore-core-4-i386'
	;;
    debian-potato-i386)
        DISTRO_EQUIV='debian-potato-i386'
        ;;
    debian-woody-i386)
        DISTRO_EQUIV='debian-woody-i386'
        ;;
    yellowdog-20-ppc)
        DISTRO_EQUIV='yellowdog-20-ppc'
        ;;
    yellowdog-21-ppc)
        DISTRO_EQUIV='yellowdog-21-ppc'
        ;;
    yellowdog-22-ppc)
        DISTRO_EQUIV='yellowdog-22-ppc'
        ;;
    redhat-Enterprise-ia64)
        DISTRO_EQUIV='redhat-Enterprise-ia64'
        ;;
    fedora-core-3-x86_64)
	DISTRO_EQUIV='fedora-core-3-x86_64'
	;;
    fedora-core-4-x86_64)
	DISTRO_EQUIV='fedora-core-4-x86_64'
	;;
    redhat-7-sparc)
	DISTRO_EQUIV='redhat-7-sparc'
	;;
    *)
        bail_unsupported $DISTRO_STRING
        ;;
esac

echo "==> Distibution: $DISTRO_EQUIV"

RPMVER=`rpm --version | awk '{print $3}'`
echo "==> The RPM Version on this machine is: $RPMVER"

case $RPMVER in
    2*)
       echo Building for RPM v2.x
       cp -f mkcdrec2.spec mkcdrec.spec
       ed mkcdrec.spec << EOF > /tmp/mkcdrec_rpm.log 2>&1
/PVERSION
s/PVERSION/${VERSION}/g
/BOOT_ARCH
s/BOOT_ARCH/${BOOTARCH}/g
w
q
EOF
      ;;
    3*)
       echo Building for RPM v3.x
       cp -f mkcdrec2.spec mkcdrec.spec
ed mkcdrec.spec << EOF > /tmp/mkcdrec_rpm.log 2>&1
/PVERSION
s/PVERSION/${VERSION}/g
/BOOT_ARCH
s/BOOT_ARCH/${BOOTARCH}/g
w
q
EOF
       ;;
    4*)
       echo Building for RPM v4.x
       cp -f mkcdrec2.spec mkcdrec.spec
      ed mkcdrec.spec << EOF > /tmp/mkcdrec_rpm.log 2>&1
/PVERSION
s/PVERSION/${VERSION}/g
/BOOT_ARCH
s/BOOT_ARCH/${BOOTARCH}/g
w
q
EOF
       ;;
    *)
       echo "Unknown RPM version: `rpm --version`"
       exit 1
       ;;
esac

case $BOOTARCH in
	x86|x86_64|ppc|sparc)
		( cd .. ; if [ -f Makefile.$BOOTARCH ]; then make -f Makefile.$BOOTARCH cvsclean; fi )

		# We do this to make sure that the package always has the current version in it''s name

		( cd .. ; if [ -f Makefile.$BOOTARCH ]; then make -f Makefile.$BOOTARCH dist; fi )
		# in ../.. we have a source tar-ball of mkCDrec_$VERSION.tar.gz
	;;
	ia64)
		( cd .. ; if [ -f Makefile.ia64 ]; then make -f Makefile.ia64 cvsclean; fi )

		# We do this to make sure that the package always has the current version in it''s name

		( cd .. ; if [ -f Makefile.ia64 ]; then make -f Makefile.ia64 dist; fi )
		# in ../.. we have a source tar-ball of mkCDrec_$VERSION.tar.gz
	;;
	*)
		echo "unknown architecture: $BOOTARCH "
		exit 1
	;;
esac

cp -v ../../mkCDrec_${VERSION}.tar.gz ${SRCDIR}

cp -av mkcdrec.spec ${SPECDIR}

echo Getting Ready to build release package
cd ${SPECDIR}
##rpm -ba -v --clean --rmsource mkcdrec.spec
rpmbuild -ba -vv mkcdrec.spec

echo Done.

